"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Log = exports.LogBase = exports.debug = void 0;
var debug_1 = __importDefault(require("debug"));
exports.debug = debug_1.default;
var path_1 = require("path");
var cleanInspectOpts_1 = require("./helpers/cleanInspectOpts");
var isPrimitive_1 = require("./helpers/isPrimitive");
var util_1 = require("util");
var callerCallsite_1 = require("./helpers/callerCallsite");
// TODO: Figure deprecation later
/**
 * @deprecated Please use `import { debug } from 'debug-next'`. Or, you may instantiate automatically namespaces loggers via `import { Log } from 'debug-next'`.
 */
exports.default = debug_1.default;
exports.LogBase = {
    appName: 'debug',
    baseDir: '',
    truncateDir: ['src/', 'dist/'],
    namespace: function (fileName) {
        var relatived = (0, path_1.relative)(this.baseDir, fileName);
        // #relative will return the same name if the directory
        // doesn't match. it is also an indication that this
        // is not a filename, but an actual namespace. just return.
        if (relatived === fileName)
            return relatived;
        for (var i = 0; i < this.truncateDir.length; i++) {
            var truncate = this.truncateDir[i];
            if (relatived.indexOf(truncate) === 0) {
                relatived = relatived.slice(truncate.length, relatived.length);
            }
        }
        if (relatived.indexOf('../') === 0) {
            return "".concat(this.appName, ":").concat(relatived.replace('../', ''));
        }
        return "".concat(this.appName, ":").concat(relatived.replace(/\//g, ':').replace(/\\/g, ':'));
    },
    _isInitialized: false,
    /**
     * Initialize LogBase. Can only be done once.
     * @param appName Name of the app. This is a prefix to the namespace. E.g. appName:namespace1:namespace2
     * @param baseDir Base directory of the app. This is required to diff the paths with other files to get a relative path for namespacing.
     */
    init: function (appName, baseDir) {
        if (this._isInitialized)
            throw Error('LogBase can only be initialised once.');
        this.appName = appName;
        this.baseDir = baseDir;
        this._isInitialized = true;
        Object.freeze(this);
        return this;
    },
    _hooks: {
        log: {},
        logWarn: {},
        logDebug: {},
        logVerbose: {},
        logError: {},
        logFatal: {},
    },
    _hooksUnsafe: {
        log: {},
        logWarn: {},
        logDebug: {},
        logVerbose: {},
        logError: {},
        logFatal: {},
    },
    _runHooks: function (logger, isEnabled, scope, args) {
        var _a, _b;
        for (var hook in exports.LogBase._hooks[logger]) {
            (_a = this._hooks[logger])[hook].apply(_a, __spreadArray([isEnabled, scope], args, false));
        }
        for (var hook in exports.LogBase._hooksUnsafe.log) {
            (_b = this._hooksUnsafe[logger])[hook].apply(_b, __spreadArray([isEnabled, scope], args, false));
        }
    },
    /**
     * Hooks added will run after the logging is complete.
     * @param attachTo Attach hook to a logger.
     * @param hookName Name of hook.
     * @param hook Hook to run. Will be try/caught and not throw.
     *
     * @example
     * // To add a transportation to Datadog
     * import { datadogLogs } from '@datadog/browser-logs'
     * const datadogHook = (args, loggerType, isEnabled, scope, hookName) => {
     *     // run only if the logger was enabled by namespace
     *     if (isEnabled) {
     *
     *         // loggerType is log/logDebug etc...
     *         const msg = loggerType
     *
     *         // scope is the function caller name
     *         msg += ` ${scope}`
     *
     *         // hookName is `datadog` in this case
     *         msg += ` ${hookName}`
     *
     *         // hookName is `datadog` in this case
     *         datadogLogs.logger.info(msg, { info: args })
     *     }
     * }
     * LogBase.addHook('log', 'datadog', datadogHook)
     *
     * @example
     * // Adding Sentry hook
     * import * as Sentry from '@sentry/node'
     * const sentryHook = (...args) => {
     *     if (args.length === 1 && args[0] instanceof Error) {
     *         Sentry.captureException(args[0])
     *         return
     *     }
     *
     *     Sentry.captureException({
     *        message: 'Error!',
     *        info: args
     *     })
     * }
     * LogBase.attachHook('logError', 'sentry', sentryHook)
     */
    attachHook: function (attachTo, hookName, hook) {
        if (attachTo === 'all') {
            ;
            [
                'log',
                'logWarn',
                'logDebug',
                'logVerbose',
                'logError',
                'logFatal',
            ].forEach(function (attachTo) { return exports.LogBase.attachHook(attachTo, hookName, hook); });
            return;
        }
        var hooks = this._hooks[attachTo];
        if (hooks[hookName]) {
            throw Error("Hook name of ".concat(hookName, " already exist for #").concat(attachTo, " method. If this is not a mistake, use another name."));
        }
        hooks[hookName] = function (isEnabled, scope) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            try {
                return hook(args, attachTo, isEnabled, scope, hookName);
            }
            catch (err) {
                var msg = "Hook `".concat(hookName, "` attached to ").concat(attachTo, " threw an error.");
                msg += " Please make sure you pass in hooks that are safe to execute. This is try/caught by debug to prevent execution errors.";
                msg += " If you need to throw, use `.addHookUnsafe`.";
                // eslint-disable-next-line no-console
                console.error(msg);
                // eslint-disable-next-line no-console
                console.error(err);
            }
        };
    },
    /**
     * An unsafe version of #attachHook. Use this if you want the failure of the hook to throw.
     * Usage is the same as #attachHook.
     */
    attachHookUnsafe: function (attachTo, hookName, hook) {
        if (attachTo === 'all') {
            ;
            [
                'log',
                'logWarn',
                'logDebug',
                'logVerbose',
                'logError',
                'logFatal',
            ].forEach(function (attachTo) { return exports.LogBase.attachHook(attachTo, hookName, hook); });
            return;
        }
        var hooks = this._hooksUnsafe[attachTo];
        if (hooks[hookName]) {
            throw Error("Hook name of ".concat(hookName, " already exist for #").concat(attachTo, " method. If this is not a mistake, use another name."));
        }
        hooks[hookName] = function (isEnabled, scope) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            return hook(args, attachTo, isEnabled, scope, hookName);
        };
    },
};
/**
 * Create a Debugger instance that will automatically turn filenames into namespaces.
 * @param filenameOrNamespace
 * @returns
 */
var createLogger = function (filenameOrNamespace) {
    var namespace = undefined;
    if (filenameOrNamespace !== undefined) {
        namespace = exports.LogBase.namespace(filenameOrNamespace);
    }
    else {
        var callsite = (0, callerCallsite_1.callerCallsite)({ depth: 0 });
        if (callsite === null || callsite === void 0 ? void 0 : callsite.file) {
            namespace = exports.LogBase.namespace(callsite === null || callsite === void 0 ? void 0 : callsite.file);
        }
    }
    // create a debugger that logs to normal logs
    var logger = (0, debug_1.default)(namespace || '');
    // overwrite old behaviour of writing to `process.stderr` to use `process.stdout` instead.
    // this allow logs to be separated between non-error types (process.stdout) and error types (process.stderr)
    logger.log = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var inspectOpts = (0, cleanInspectOpts_1.cleanInspectOpts)(logger.inspectOpts);
        return process.stdout.write("".concat(util_1.formatWithOptions.apply(void 0, __spreadArray([inspectOpts], args, false)), "\n"));
    };
    return logger;
};
var _createDebugger = function (name, debugInstance) {
    var modifiedDebugger = Object.assign(function () {
        var _a;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        // callerCallsite has some performance overheads.
        // if it not enabled, and have no hooks to run, return
        if (!debugInstance.enabled &&
            Object.keys(exports.LogBase._hooks[name]).length === 0 &&
            Object.keys(exports.LogBase._hooksUnsafe[name]).length === 0) {
            return;
        }
        // get callsite, run debug, and hooks afterwards.
        var scope = (_a = (0, callerCallsite_1.callerCallsite)({ depth: 0 })) === null || _a === void 0 ? void 0 : _a.scope;
        debugWithScope.apply(void 0, __spreadArray([scope || '', debugInstance], args, false));
        exports.LogBase._runHooks(name, debugInstance.enabled, scope, args);
    }, debugInstance);
    // restore the reference of `enabled` back to the Debugger
    Object.defineProperty(modifiedDebugger, 'enabled', {
        get: function () {
            return debugInstance.enabled;
        },
        set: function (v) {
            debugInstance.enabled = v;
        },
    });
    return modifiedDebugger;
};
/**
 * Call this to debug with scope. Scope can either be passed in, or be automagically obtained.
 * @param scope Pass in scope if you have, otherwise use null/undefined to automagically obtain scope. Pass in empty string '' skip (no scope).
 * @param logger A debugger to use.
 * @param args Arguments to be passed to the debugger.
 */
var debugWithScope = function (scope, logger) {
    var _a;
    var args = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        args[_i - 2] = arguments[_i];
    }
    var gotScope = scope !== null && scope !== void 0 ? scope : (_a = (0, callerCallsite_1.callerCallsite)({ depth: 0 })) === null || _a === void 0 ? void 0 : _a.scope;
    // if scope is empty string, undefined, or null
    // log without the need to switch the scope
    if (!gotScope)
        return logger.apply(void 0, args);
    // the reason why we needed to switch scope this way
    // is because the debugger instance is shared by all scopes in the same namespace
    // so we switch it only for calling the debugger, and restore after
    logger.scope = gotScope;
    var d = logger.apply(void 0, args);
    logger.scope = null;
    return d;
};
/**
 * A centralised logging interface that will create namespaced debugger automatically using the file structure.
 * @param fileName [optional] If left empty, it will Pass in `__filename`.
 */
var Log = function (fileName) {
    // create a debugger that logs to normal logs
    var debugStdOut = createLogger(fileName);
    // create a debugger that logs to error logs (default behaviour of the debug module)
    var debugStdErr = (0, debug_1.default)(debugStdOut.namespace);
    // force debug to be enabled despite namespacing
    if (process.env.LOG_ERROR === 'true' || process.env.DEBUG_ERROR === 'true')
        debugStdErr.enabled = true;
    // only overwrite when explicitly declared to be false
    if (process.env.LOG_ERROR === 'false' || process.env.DEBUG_ERROR === 'false')
        debugStdErr.enabled = false;
    // if debugStdErr is enabled, we can use the same debugger for FATAL
    var debugStdErrFatal;
    if (debugStdErr.enabled === true) {
        debugStdErrFatal = debugStdErr;
    }
    else {
        // else we will create another one.
        debugStdErrFatal = (0, debug_1.default)(debugStdOut.namespace);
        // fatal is forced enabled by default
        debugStdErrFatal.enabled = true;
    }
    var log = _createDebugger('log', debugStdOut);
    var logWarn = _createDebugger('logWarn', debugStdOut);
    var logDebug = _createDebugger('logDebug', debugStdOut);
    var logVerbose = Object.assign(function () {
        var _a, _b;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        // callerCallsite has some performance overheads.
        // if it not enabled, and have no hooks to run, return
        if (!debugStdOut.enabled &&
            Object.keys(exports.LogBase._hooks.logVerbose).length === 0 &&
            Object.keys(exports.LogBase._hooksUnsafe.logVerbose).length === 0) {
            return;
        }
        // LOG_VERBOSE for backward compatibility
        // use DEBUG_VERBOSE
        if (process.env.LOG_EXPAND_VERBOSE) {
            // eslint-disable-next-line no-console
            console.warn('Deprecated: Use `DEBUG_VERBOSE` instead of `LOG_EXPAND_VERBOSE`.');
        }
        if (
        // If verbose mode is on, log normally.
        process.env.LOG_EXPAND_VERBOSE === 'true' ||
            process.env.DEBUG_VERBOSE === 'true' ||
            // if there is only 1 argument of primitive type
            (args.length === 1 && (0, isPrimitive_1.isPrimitive)(args[0])) ||
            // if there are 2, and both are primitives
            (args.length === 2 && (0, isPrimitive_1.isPrimitive)(args[0]) && (0, isPrimitive_1.isPrimitive)(args[1]))) {
            var scope_1 = (_a = (0, callerCallsite_1.callerCallsite)({ depth: 0 })) === null || _a === void 0 ? void 0 : _a.scope;
            debugWithScope.apply(void 0, __spreadArray([scope_1 || '', debugStdOut], args, false));
            exports.LogBase._runHooks('logVerbose', debugStdOut.enabled, scope_1, args);
            return;
        }
        var messageIfFirstArgumentIsString = '';
        var message = '';
        // copy out args array before it gets manipulated.
        // _runHooks needs to call it later
        var copiedArgs = null;
        // if the first argument is a string, we will use it as the first message
        if (typeof args[0] === 'string') {
            // copy out args array before it gets manipulated.
            // _runHooks needs to call it later
            copiedArgs = __spreadArray([], args, true);
            messageIfFirstArgumentIsString = args.shift();
        }
        if (args[0] === null || args[0] === undefined) {
            message = typeof args[0];
        }
        else if (Array.isArray(args[0])) {
            message = "an array of length ".concat(args[0].length);
        }
        else if (typeof args[0] === 'object') {
            try {
                message = "an object with keys [".concat(Object.keys(args[0]), "]");
            }
            catch (e) {
                message = "an unknown object";
            }
        }
        else {
            message = 'an unknown object';
        }
        if (args.length > 1)
            message += " (and ".concat(args.length - 1, " more argument").concat(args.length - 1 === 1 ? '' : 's', ").");
        var finalMessage = messageIfFirstArgumentIsString;
        if (messageIfFirstArgumentIsString && message)
            finalMessage += ' | ';
        if (message) {
            finalMessage += "Verbose debugger available for: ".concat(message);
        }
        var scope = (_b = (0, callerCallsite_1.callerCallsite)({ depth: 0 })) === null || _b === void 0 ? void 0 : _b.scope;
        debugWithScope(scope || '', debugStdOut, finalMessage);
        exports.LogBase._runHooks('logVerbose', debugStdOut.enabled, scope, copiedArgs || args);
        return;
    }, debugStdOut);
    var logError = _createDebugger('logError', debugStdErr);
    var logFatal = Object.assign(function () {
        var _a;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var scope = (_a = (0, callerCallsite_1.callerCallsite)({ depth: 0 })) === null || _a === void 0 ? void 0 : _a.scope;
        debugWithScope.apply(void 0, __spreadArray(__spreadArray([scope || '',
            debugStdErrFatal,
            '\x1b[31m',
            'FATAL:'], args, false), ['\u001B[0m'], false));
        exports.LogBase._runHooks('logFatal', debugStdErrFatal.enabled, scope, args);
    }, debugStdErrFatal);
    return { log: log, logWarn: logWarn, logError: logError, logFatal: logFatal, logDebug: logDebug, logVerbose: logVerbose };
};
exports.Log = Log;
//# sourceMappingURL=index.js.map